/*!
 * \file    selection.h
 * \brief   Template meta-programming type selections.
 *
 * \copyright
 *    Copyright (C) 2018 Christos Choutouridis <christos@choutouridis.net>\n
 *    This program is free software: you can redistribute it and/or modify
 *    it under the terms of the GNU Lesser General Public License as
 *    published by the Free Software Foundation, either version 3
 *    of the License, or (at your option) any later version.\n
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU Lesser General Public License for more details.\n
 *    You should have received a copy of the GNU Lesser General Public License
 *    along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
#ifndef __utl_meta_selection_h__
#define __utl_meta_selection_h__

#include <utl/core/impl.h>
#include <utl/meta/basic.h>

/*!
 * \ingroup meta
 * \defgroup selection Selection
 * Type selection support header
 */
//! @{
namespace utl {
namespace meta{

   //! \name if implementation
   //! @{
   namespace details {
      template <bool If, typename...>
      struct if_c_ {
         using type = nil_;   //< avoid ill formed result
      };
      template<typename Then>
      struct if_c_<true, Then> {
         using type = Then;
      };
      template<typename Then, typename Else>
      struct if_c_<true, Then, Else> {
         using type = Then;
      };
      template<typename Then, typename Else>
      struct if_c_<false, Then, Else> {
         using type = Else;
      };
   }
   //! Select one type or another depending on a compile-time Boolean.
   template <bool B, typename... Args>
   using if_c = eval_t<details::if_c_<B, Args...>>;

   //! Select one type or another depending on a compile-time Boolean type
   template <typename If, typename... Args>
   using if_ = if_c<If::type::value, Args...>;

   //! @}

   /*!
    * \name Named type selectors
    */
   //! @{

   //! Select the first type of a type sequence
   template <typename T1, typename ...> using first_of = T1;

   //! Select the second type of a type sequence
   template <typename T1, typename T2, typename ...> using second_of = T2;

   //! @}
}}

//! @}

#endif /* __utl_meta_selection_h__ */
