/*!
 * \file    sfinae.h
 * \brief   Template meta-programming SFINAE helpers
 *
 * \copyright
 *    Copyright (C) 2018 Christos Choutouridis <christos@choutouridis.net>\n
 *    This program is free software: you can redistribute it and/or modify
 *    it under the terms of the GNU Lesser General Public License as
 *    published by the Free Software Foundation, either version 3
 *    of the License, or (at your option) any later version.\n
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU Lesser General Public License for more details.\n
 *    You should have received a copy of the GNU Lesser General Public License
 *    along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
#ifndef __utl_meta_sfinae_h__
#define __utl_meta_sfinae_h__

#include <utl/core/impl.h>
#include <type_traits>

/*!
 * \ingroup meta
 * \defgroup sfinae sfinae
 * conditional use support header.
 */
//! @{
namespace utl {
namespace meta {

   //! \name when implementation
   //! @{
   namespace details {
//      template <typename... T>
//      struct dev_null { using type = dev_null; };  //< Same as typelist

      template <bool If>  struct when_ { };
      template <>         struct when_<true> { using type = void; };
   }
   //! Tool to enable a partial specialization only if a boolean condition is true.
   //! Well formed only if \p If is true
   template <bool If>
   using when = eval_t< details::when_<If> >;

//   //! Well formed only if all of \p Ifs are \c true
//   template <bool ...Ifs>
//   using when_all = details::dev_null<
//         when<Ifs>...
//   >;

   //! @}

   //! \name enable_if from STL
   //! @{

   //! enable_if, imported from stl
   template <bool If, typename _Tp = void>   using enable_if = std::enable_if<If, _Tp>;

   //! alias template for enable_if
   template<bool If, typename _Tp = void>    using enable_if_t = eval_t< enable_if<If, _Tp> >;

   //! @}

}}

//! @}

#endif /* __utl_meta_sfinae_h__ */
